@@ -1,4 +1,4 @@
-__version__ = '0.0.36'
+__version__ = '0.0.37'
 
 DB_VERSION = 1
 
@@ -6,8 +6,12 @@ DEFAULT_FMT = 'json'
 
 DEFAULT_HTTP_TIMEOUT = 5
 
+DEFAULT_SOCKET_TIMEOUT = 5
+
 DEFAULT_CACHE_SIZE = 1000
 
+SOCKET_BUF = 8192
+
 FMTS = ['json', 'yaml', 'msgpack', 'cbor', 'pickle']
 
 import threading
@@ -22,6 +26,7 @@ from functools import partial
 from cachetools import LRUCache
 
 import os
+import socket
 
 import logging
 
@@ -61,20 +66,33 @@ class YEDB():
     The object is thread-safe
     """
 
+    def _init_socket(self):
+        db_socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+        db_socket.settimeout(self.socket_timeout)
+        db_socket.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 8192)
+        db_socket.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192)
+        return db_socket
+
     def _remote_call(self, method, **kwargs):
-        import uuid
-        req = {
-            'jsonrpc': '2.0',
-            'id': str(uuid.uuid4()),
-            'method': method,
-            'params': kwargs
-        }
+
+        def _reopen_socket():
+            if debug:
+                logger.debug(f'(re)opening socket {self.db}')
+            db_socket = self._init_socket()
+            db_socket.connect(self.db)
+            g.db_socket = db_socket
+            return db_socket
+
+        req = {'jsonrpc': '2.0', 'id': 1, 'method': method, 'params': kwargs}
         try:
             import msgpack
             use_msgpack = True
             data = msgpack.dumps(req)
-            headers = {'Content-Type': 'application/x-msgpack'}
+            if not self.use_db_socket:
+                headers = {'Content-Type': 'application/x-msgpack'}
         except ModuleNotFoundError:
+            if self.use_db_socket:
+                raise
             try:
                 import rapidjson as json
             except:
@@ -85,25 +103,51 @@ class YEDB():
         if debug:
             logger.debug(f'JRPC ({"msgpack" if use_msgpack else "json"}) '
                          f'{self.db} method={method} auth={self.http_auth}')
-        try:
-            post = g.session.post
-        except AttributeError:
-            import requests
-            session = requests.Session()
-            g.session = session
-            post = session.post
-        # from requests import post
-        r = post(self.db,
-                 data=data,
-                 headers=headers,
-                 timeout=self.http_timeout,
-                 auth=self.http_auth)
-        if not r.ok:
-            raise RuntimeError(f'http response code {r.status_code}')
-        if use_msgpack:
-            data = msgpack.loads(r.content, raw=False)
+        if self.use_db_socket:
+            try:
+                db_socket = g.db_socket
+            except AttributeError:
+                db_socket = _reopen_socket()
+            with self.lock:
+                if db_socket._closed:
+                    db_socket = _reopen_socket()
+                try:
+                    try:
+                        db_socket.sendall(
+                            len(data).to_bytes(4, 'little') + data)
+                    except BrokenPipeError:
+                        db_socket = _reopen_socket()
+                        db_socket.sendall(
+                            len(data).to_bytes(4, 'little') + data)
+                    frame = db_socket.recv(4)
+                    frame_len = int.from_bytes(frame, 'little')
+                    response = b''
+                    while len(response) < frame_len:
+                        response += db_socket.recv(SOCKET_BUF)
+                    data = msgpack.loads(response, raw=False)
+                except:
+                    db_socket.close()
+                    raise
         else:
-            data = json.loads(r.text)
+            try:
+                post = g.session.post
+            except AttributeError:
+                import requests
+                session = requests.Session()
+                g.session = session
+                post = session.post
+            # from requests import post
+            r = post(self.db,
+                     data=data,
+                     headers=headers,
+                     timeout=self.http_timeout,
+                     auth=self.http_auth)
+            if not r.ok:
+                raise RuntimeError(f'http response code {r.status_code}')
+            if use_msgpack:
+                data = msgpack.loads(r.content, raw=False)
+            else:
+                data = json.loads(r.text)
         try:
             error_code = data['error']['code']
             if error_code == -32001:
@@ -123,12 +167,15 @@ class YEDB():
     def _not_implemented(self, *args, **kwargs):
         raise RuntimeError('not implemented in remote mode')
 
-    def _test_remote(self):
+    def _open_remote(self, **kwargs):
         result = self._remote_call('test')
         if result.get('name') != 'yedb':
             raise RuntimeError('unsupported RPC server')
         return result
 
+    def _close_remote(self):
+        pass
+
     def __init__(
         self,
         dbpath,
@@ -169,15 +216,23 @@ class YEDB():
             cache_size: item cache size
         """
         path = str(dbpath)
+        self.lock = RLock()
         self.auto_repair = kwargs.get('auto_repair')
         self.cache = LRUCache(kwargs.get('cache_size', DEFAULT_CACHE_SIZE))
+        self.timeout = 5
         if debug:
             logger.debug('initializing db')
             logger.debug(f'path: {path}')
             logger.debug(f'options: {kwargs}')
-        if path.startswith('http://') or path.startswith('https://'):
+        if path.startswith('http://') or path.startswith('https://') or Path(
+                path).is_socket() or path.endswith('.sock') or path.endswith(
+                    '.socket'):
             self.db = path
+            self.use_db_socket = Path(path).is_socket() or path.endswith(
+                '.sock') or path.endswith('.socket')
             self.http_timeout = kwargs.get('http_timeout', DEFAULT_HTTP_TIMEOUT)
+            self.socket_timeout = kwargs.get('socket_timeout',
+                                             DEFAULT_SOCKET_TIMEOUT)
             username = kwargs.get('http_username')
             if username:
                 password = kwargs.get('http_password', '')
@@ -196,9 +251,9 @@ class YEDB():
                 fn = getattr(self, f)
                 if fn.__class__.__name__ == 'method':
                     if f == 'open':
-                        setattr(self, f, self._test_remote)
+                        setattr(self, f, self._open_remote)
                     elif f == 'close':
-                        setattr(self, f, self._empty)
+                        setattr(self, f, self._close_remote)
                     elif not f.startswith('_'):
                         if f in METHODS:
                             setattr(self, f, partial(self._remote_call, f))
@@ -209,7 +264,6 @@ class YEDB():
             self.default_fmt = default_fmt if default_fmt else DEFAULT_FMT
             self.default_checksums = default_checksums
             self._dbpath_len = len(self.db.absolute().as_posix()) + 1
-            self.lock = RLock()
             self._key_locks = {}
             self._opened = False
             self._flock = None
