@@ -1,4 +1,4 @@
-__version__ = '0.0.36'
+__version__ = '0.0.37'
 
 PID_FILE = '/tmp/yedb-server.pid'
 
@@ -16,7 +16,7 @@ import logging
 
 logger = logging.getLogger('yedb')
 
-if os.getenv('DEBUG'):
+if os.getenv('DEBUG') == '1':
     import logging
     logging.basicConfig(level=logging.DEBUG)
     yedb.debug = True
